from django.db import models

class AllowedContentType(models.Model):

    allowed_type = models.CharField(
        max_length = 255,
        verbose_name = 'Content-Type',
        unique = True)

    def __unicode__(self):
        return self.allowed_type

    class Meta:
        verbose_name = 'Allowed Content Type'
        ordering = ('allowed_type',)



import pyclamd, tempfile
from django.test import TestCase, Client
from clamav_upload.models import AllowedContentType

class AllowedContentTypeTestCase(TestCase):

    def setUp(self):
        self.client = Client()
        self.cd = pyclamd.ClamdAgnostic()
        self.no_eicar = tempfile.NamedTemporaryFile()
        self.no_eicar.write('no virus in this file')
        self.no_eicar.flush()
        self.no_eicar.seek(0)
        self.eicar = tempfile.NamedTemporaryFile()
        self.eicar.write(self.cd.EICAR())
        self.eicar.flush()
        self.eicar.seek(0)
        self.allowed_type = AllowedContentType(
            allowed_type='text/plain')
        self.allowed_type.save()

    def tearDown(self):
        self.eicar.close()
        self.no_eicar.close()
        try:
            self.allowed_type.delete()
        except AssertionError:
            pass

    def test_eicar(self):
        eicar = self.cd.scan_stream(self.eicar.read())
        no_eicar = self.cd.scan_stream(self.no_eicar.read())
        self.assertEqual(eicar.get(
            'stream', None), ('FOUND', 'Eicar-Test-Signature'))
        self.assertEqual(no_eicar, None)

    def test_disallowed_content_types(self):
        self.allowed_type.delete()
        self.assertEqual(
            self.client.post('/upload/', {'file': self.no_eicar}).status_code, 403)
        
    def test_upload_allowed_content_types(self):
        self.assertEqual(
            self.client.post('/upload/', {'file': self.no_eicar}).status_code, 200)

    def test_upload_virus_signature_detection(self):
        self.assertEqual(
            self.client.post('/upload/', {'file': self.eicar}).status_code, 403)



from django.contrib import messages
from django.core.exceptions import PermissionDenied


class UploadPermissionDenied(PermissionDenied):

    def __init__(self, request, log_func, error_message, *args, **kwargs):
        log_func(error_message)
        messages.error(request, error_message)
        super(UploadPermissionDenied, self).__init__(*args, **kwargs)



import pyclamd, logging, magic
from .models import AllowedContentType
from clamav_upload import get_settings
from .exceptions import UploadPermissionDenied
from django.core.files.uploadhandler import TemporaryFileUploadHandler


logger = logging.getLogger(__name__)
s = get_settings()


class ClamAVFileUploadHandler(TemporaryFileUploadHandler):

    def __init__(self, *args, **kwargs):
        super(ClamAVFileUploadHandler, self).__init__(*args, **kwargs)
        self.content_type_check = s.get('CONTENT_TYPE_CHECK_ENABLED')
        self.content_type_already_checked = False
        self.is_last_handler = False
        if s.get('LAST_HANDLER') == "{0}.{1}".format(__name__, self.__class__.__name__):
            self.is_last_handler = True
        try:
            self.cd = pyclamd.ClamdAgnostic()
        except ValueError:
            raise UploadPermissionDenied(self.request, logger.critical, 'Service currently unavailable')

    def new_file(self, file_name, *args, **kwargs):
        super(ClamAVFileUploadHandler, self).new_file(file_name, *args, **kwargs)
        logger.info('Starting new file upload, scanning for malicious content')
        logger.debug('Original Filename: {0}'.format(self.file.name))
        logger.debug('Temporary Filepath: {0}'.format(self.file.temporary_file_path()))

    def check_content_type(self, upload_buffer):
        if self.content_type_check and not self.content_type_already_checked:
            content_type = None
            try:
                content_type = magic.from_buffer(upload_buffer, mime=True)
                if content_type is None:
                    raise UploadPermissionDenied(
                        self.request, logger.critical, 'Unable to determine content-type, upload denied!')
                AllowedContentType.objects.get(allowed_type=content_type)
                self.content_type_already_checked = True
                logger.debug('Content-Type: "{0}"'.format(content_type))
            except AllowedContentType.DoesNotExist:
                    raise UploadPermissionDenied(
                        self.request, logger.critical, 'Content-Type: {0} is not an accepted type'.format(content_type))

    def receive_data_chunk(self, raw_data, start):
        self.check_content_type(raw_data)
        try:
            if self.cd.scan_stream(raw_data) is None:
                if self.is_last_handler:
                    self.file.write(raw_data)
                else:
                    return raw_data
            else:
                raise UploadPermissionDenied(
                    self.request, logger.critical, 'Malicious content detected in stream, upload denied!')
        except pyclamd.ConnectionError as e:
            raise UploadPermissionDenied(self.request, logger.critical, e.message)

    def file_complete(self, file_size):
        logger.info('File upload: "{0}" complete!'.format(self.file.name))
        if self.is_last_handler:
            self.file.seek(0)
            self.file.size = file_size
            return self.file



from django.conf import settings
"""
    Configuration options for the ``clamav_upload`` package:

    All options reside in a dictionary, ``CLAMAV_UPLOAD`` within your project's ``settings.py``. The options are as
    follows:

        CONTENT_TYPE_CHECK_ENABLED      Set to ``True`` or ``False`` in order to enable or disable content type checking
                                        via python-magic. Default is ``False``.

    To configure any or all of these options, you simply need to define them in ``settings.py``:

        CLAMAV_UPLOAD = {
            'CONTENT_TYPE_CHECK_ENABLED': True,
        }

"""

def get_settings():
    """
        This function returns a dict containing default settings
    """
    s = getattr(settings, 'CLAMAV_UPLOAD', {})
    s = {
            'CONTENT_TYPE_CHECK_ENABLED': s.get('CONTENT_TYPE_CHECK_ENABLED', False),
            # LAST_HANDLER is not a user configurable option; we return
            # it with the settings dict simply because it's convenient.
            'LAST_HANDLER': getattr(settings, 'FILE_UPLOAD_HANDLERS')[-1]
        }
    return s




import os, sys
DEBUG = True
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
SECRET_KEY = 'fake-key'
INSTALLED_APPS = [
    'tests',
    'clamav_upload'
]
ROOT_URLCONF = 'tests.urls'

MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
)

FILE_UPLOAD_HANDLERS = ( 
    'clamav_upload.handlers.ClamAVFileUploadHandler',
)

DATABASES = { 
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'test.db.sqlite3'),
    }
}

MEDIA_ROOT = os.path.join(BASE_DIR, 'test_uploaded_files_delete_me')

CLAMAV_UPLOAD = {
    'CONTENT_TYPE_CHECK_ENABLED': True
}

LOGGING = {
    'version': 1,
    'disable_existing_loggers': True,
    'formatters': {
        'standard': {
            'format': '[%(asctime)s] [THIS IS A TEST] [%(levelname)s] [%(name)s] %(message)s'
        }
    },
    'handlers': {
        'default': {
            'level': 'DEBUG',
            'class': 'logging.handlers.RotatingFileHandler',
            'filename': '/tmp/clamav_upload.log',
            'maxBytes': 1024*1024*20,
            'backupCount': 30,
            'formatter': 'standard',
        }
    },
    'loggers': {
        '': {
            'handlers': ['default'],
            'level': 'DEBUG',
            'propogate': True
        }
    }
}



from distutils.core import setup

setup(
    name='django-clamav-upload',
    version='0.1.2',
    packages=['clamav_upload'],
    url='https://github.com/musashiXXX/django-clamav-upload',
    download_url='https://github.com/musashiXXX/django-clamav-upload/tarball/0.1.2',
    license='LICENSE.txt',
    author='Charles Hamilton',
    author_email='musashi@nefaria.com',
    description='A simple upload handler for django that scans inbound streams for malicious content',
    long_description='README.md',
    install_requires=[
        'Django>=1.8.4',
        'pyClamd==0.3.17'
    ],
    keywords='clamav django upload'
)



from .forms import FileUploadForm
from django.http import HttpResponse, HttpResponseServerError

def file_upload(request):
    upload_form = FileUploadForm(request.POST, request.FILES)
    if upload_form.is_valid():
        upload_form.save()
    return HttpResponse()
